==> webform_booking-1.0.1.tar.gz: webform_booking/src/Controller/WebformBookingController.php <==
<?php

namespace Drupal\webform_booking\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\webform\Entity\Webform;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for Webform Booking.
 */
class WebformBookingController extends ControllerBase {

  /**
   * Get available days.
   */
  public function getAvailableDays($webform_id, $date) {
    $webform = Webform::load($webform_id);
    $elements = $webform->getElementsDecodedAndFlattened();
    $config = [];

    foreach ($elements as $element) {
      if (isset($element['#type']) && $element['#type'] === 'webform_booking') {
        $config = $element;
        break;
      }
    }

    $exclusionPatterns = explode("\n", $config['#exclusion_dates']);
    $excludedWeekdays = $config['#excluded_weekdays'];

    // Prepare the exclusion dates, including intervals.
    $exclusionDates = [];
    foreach ($exclusionPatterns as $pattern) {
      if (strpos($pattern, '|') !== FALSE) {
        [$start, $end] = explode('|', $pattern);
        $period = new \DatePeriod(
        new \DateTime($start),
        new \DateInterval('P1D'),
        (new \DateTime($end))->modify('+1 day')
        );
        foreach ($period as $d) {
          $exclusionDates[] = $d->format('Y-m-d');
        }
      }
      else {
        $exclusionDates[] = $pattern;
      }
    }

    // Parse the input date to get the start and end dates of its month.
    $inputDate = new \DateTime($date);
    $startOfMonth = $inputDate->modify('first day of this month')->setTime(0, 0, 0);
    $endOfMonth = (clone $startOfMonth)->modify('last day of this month')->setTime(23, 59, 59);

    // Interval of 1 day.
    $interval = new \DateInterval('P1D');
    $dateRange = new \DatePeriod($startOfMonth, $interval, $endOfMonth);

    $days = [];

    foreach ($dateRange as $day) {
      $formattedDay = $day->format('Y-m-d');
      $today = new \DateTime();
      $weekday = $day->format('D');
      if ($day >= $today && !in_array($formattedDay, $exclusionDates) && isset($excludedWeekdays[$weekday]) && $excludedWeekdays[$weekday] === 0) {
        $days[] = $formattedDay;
      }
    }

    return new JsonResponse($days);
  }

  /**
   * Get available slots.
   */
  public function getAvailableSlots($webform_id, $date) {

    $webform = Webform::load($webform_id);
    $elements = $webform->getElementsDecodedAndFlattened();
    $webform_key = NULL;
    $config = [];

    foreach ($elements as $key => $element) {
      if (isset($element['#type']) && $element['#type'] === 'webform_booking') {
        $config = $element;
        $webform_key = $key;
        break;
      }
    }
    $timeIntervals = explode("\n", $config['#time_interval']);
    $slotDuration = $config['#slot_duration'];

    $availableSlots = [];

    // Loop through each time interval.
    foreach ($timeIntervals as $interval) {
      [$startTimeStr, $endTimeStr] = explode('|', $interval);
      $startTime = new \DateTime($startTimeStr);
      $endTime = new \DateTime($endTimeStr);

      $currentTime = clone $startTime;

      while ($currentTime < $endTime) {
        $endSlotTime = clone $currentTime;
        $endSlotTime->add(new \DateInterval('PT' . $slotDuration . 'M'));
        if ($endSlotTime > $endTime) {
          // Ensures slot does not extend beyond the interval end time.
          break;
        }
        $timeRange = $currentTime->format('H:i') . '-' . $endSlotTime->format('H:i');
        $slot = $date . ' ' . $currentTime->format('H:i');
        $slotStatus = $this->isSlotSelected($webform_id, $webform_key, $slot) ? 'unavailable' : 'available';
        $availableSlots[] = ['time' => $timeRange, 'status' => $slotStatus];
        $currentTime = clone $endSlotTime;
      }
    }

    return new JsonResponse($availableSlots);
  }

  /**
   * Check if the slot is booked.
   */
  protected function isSlotSelected($webform_id, $webform_key, $slot) {
    $query = \Drupal::service('database')
      ->select('webform_submission_data', 'wsd')
      ->fields('wsd', ['sid'])
      ->condition('wsd.webform_id', $webform_id, '=')
      ->condition('wsd.name', $webform_key, '=')
      ->condition('wsd.value', $slot, '=')
      ->execute();
    $count = count($query->fetchAll(\PDO::FETCH_COLUMN));

    if ($count > 0) {
      return TRUE;
    }
    return FALSE;
  }

}

==> webform_booking-1.0.2.tar.gz: webform_booking/src/Controller/WebformBookingController.php <==
<?php

namespace Drupal\webform_booking\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\webform\Entity\Webform;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for Webform Booking.
 */
class WebformBookingController extends ControllerBase {

  /**
   * Get available days.
   */
  public function getAvailableDays($webform_id, $date) {
    $webform = Webform::load($webform_id);
    $elements = $webform->getElementsDecodedAndFlattened();
    $config = [];

    foreach ($elements as $element) {
      if (isset($element['#type']) && $element['#type'] === 'webform_booking') {
        $config = $element;
        break;
      }
    }

    $exclusionPatterns = explode("\n", $config['#exclusion_dates']);
    $excludedWeekdays = $config['#excluded_weekdays'];

    // Prepare the exclusion dates, including intervals.
    $exclusionDates = [];
    foreach ($exclusionPatterns as $pattern) {
      if (strpos($pattern, '|') !== FALSE) {
        [$start, $end] = explode('|', $pattern);
        $period = new \DatePeriod(
        new \DateTime($start),
        new \DateInterval('P1D'),
        (new \DateTime($end))->modify('+1 day')
        );
        foreach ($period as $d) {
          $exclusionDates[] = $d->format('Y-m-d');
        }
      }
      else {
        $exclusionDates[] = $pattern;
      }
    }

    // Parse the input date to get the start and end dates of its month.
    $inputDate = new \DateTime($date);
    $startOfMonth = $inputDate->modify('first day of this month')->setTime(0, 0, 0);
    $endOfMonth = (clone $startOfMonth)->modify('last day of this month')->setTime(23, 59, 59);

    // Interval of 1 day.
    $interval = new \DateInterval('P1D');
    $dateRange = new \DatePeriod($startOfMonth, $interval, $endOfMonth);

    $days = [];

    foreach ($dateRange as $day) {
      $formattedDay = $day->format('Y-m-d');
      $today = new \DateTime();
      $weekday = $day->format('D');
      if ($day >= $today && !in_array($formattedDay, $exclusionDates) && isset($excludedWeekdays[$weekday]) && $excludedWeekdays[$weekday] === 0) {
        $days[] = $formattedDay;
      }
    }

    return new JsonResponse($days);
  }

  /**
   * Get available slots.
   */
  public function getAvailableSlots($webform_id, $date) {

    $webform = Webform::load($webform_id);
    $elements = $webform->getElementsDecodedAndFlattened();
    $webform_key = NULL;
    $config = [];

    foreach ($elements as $key => $element) {
      if (isset($element['#type']) && $element['#type'] === 'webform_booking') {
        $config = $element;
        $webform_key = $key;
        break;
      }
    }
    $timeIntervals = explode("\n", $config['#time_interval']);
    $slotDuration = $config['#slot_duration'];
    $seats = $config['#seats_slot'] ?? 1;

    $availableSlots = [];

    // Loop through each time interval.
    foreach ($timeIntervals as $interval) {
      [$startTimeStr, $endTimeStr] = explode('|', $interval);
      $startTime = new \DateTime($startTimeStr);
      $endTime = new \DateTime($endTimeStr);

      $currentTime = clone $startTime;

      while ($currentTime < $endTime) {
        $endSlotTime = clone $currentTime;
        $endSlotTime->add(new \DateInterval('PT' . $slotDuration . 'M'));
        if ($endSlotTime > $endTime) {
          // Ensures slot does not extend beyond the interval end time.
          break;
        }
        $timeRange = $currentTime->format('H:i') . '-' . $endSlotTime->format('H:i');
        $slot = $date . ' ' . $currentTime->format('H:i');
        $slotStatus = $this->isSlotSelected($webform_id, $webform_key, $slot, $seats) ? 'unavailable' : 'available';
        $availableSlots[] = ['time' => $timeRange, 'status' => $slotStatus];
        $currentTime = clone $endSlotTime;
      }
    }

    return new JsonResponse($availableSlots);
  }

  /**
   * Check if the slot is booked.
   */
  protected function isSlotSelected($webform_id, $webform_key, $slot, $seats) {
    $query = \Drupal::service('database')
      ->select('webform_submission_data', 'wsd')
      ->fields('wsd', ['sid'])
      ->condition('wsd.webform_id', $webform_id, '=')
      ->condition('wsd.name', $webform_key, '=')
      ->condition('wsd.value', $slot, '=')
      ->execute();
    $count = count($query->fetchAll(\PDO::FETCH_COLUMN));

    if ($count < $seats) {
      return FALSE;
    }
    return TRUE;
  }

}
